/*

	Common library file, used by (at least) fine-tune.js and wound-size.js

*/

var cv = require('opencv');
var fs = require('fs');
var upath = require('upath');
var queryString = require('querystring');
var path = require('path');
var area = require('area-polygon');
var resolve = require('path').resolve;
var polygon = require('@turf/helpers').polygon;
var intersect = require('@turf/intersect').default;

var verbose = true;
var globalId = "";


var masterConfigFile = __dirname + '/../config/master.json';
var mainMedImagePath = __dirname + "/../../../photos";   //If run from two dirs in: "../../photos";
var globalConfigFile = __dirname + '/../../../config.json';






module.exports.calculateDrawingArea = function(config)
{
	
	var totalWoundSquarePixels = 0;
	var totalWoundSquareCm = 0;
	
	var totalUnderSquarePixels = 0;
	var totalUnderSquareCm = 0;
	
	var totalStickerSquarePixels = 0;


	for(var cnt = 0; cnt< config.output.areas.length; cnt++) {


		if(config.output.areas[cnt].type == "sticker") {
			if(verbose == true) console.log(JSON.stringify(config.output.areas[cnt].points));
			var areaSquarePixels = area(config.output.areas[cnt].points);
			totalStickerSquarePixels += areaSquarePixels;
		}
		
	}

	if(totalStickerSquarePixels != 0) {
		//Only override the auto-calculated option if there are sticker entries (i.e. not 0)
		config.output.stickerSquarePixels = totalStickerSquarePixels;
	}


	var woundLayer = [];
	var underLayer = [];

	for(var cnt = 0; cnt< config.output.areas.length; cnt++) {

		//Finds the areas in square pixels and square cm.
		if(config.output.areas[cnt].points) {
			if(verbose == true) console.log("Calcing area of:" + JSON.stringify(config.output.areas[cnt].points));
			var areaSquarePixels = area(config.output.areas[cnt].points);
			if(verbose == true) console.log("Area:" + areaSquarePixels);
		} else {
			var areaSquarePixels = 0;
		}


		config.output.areas[cnt].areaSquarePixels = areaSquarePixels;
		if(config.output.stickerSquarePixels) {
			var areasSquareCm = (areaSquarePixels / config.output.stickerSquarePixels) * config.input.sticker.physicalAreaSquareCm;
			config.output.areas[cnt].areaSquareCm = areasSquareCm.toFixed(3);
		} else {
			config.output.areas[cnt].areaSquareCm = 0;
		}

		//Set in the config file
		if(config.output.areas[cnt].type == "wound") {
			totalWoundSquarePixels += areaSquarePixels;
			totalWoundSquareCm += areasSquareCm;
			woundLayer.push(cnt);
		}
		
		if(config.output.areas[cnt].type == "under") {
			totalUnderSquarePixels += areaSquarePixels;
			totalUnderSquareCm += areasSquareCm;
			underLayer.push(cnt);
		}
		
		
		if(config.output.areas[cnt].type == "negative") {
			totalWoundSquarePixels -= areaSquarePixels;
			totalWoundSquareCm -= areasSquareCm;
		}
		
	}

	config.output.fullWound.areaSquarePixels = totalWoundSquarePixels;
	if((totalWoundSquareCm == 0)||(totalStickerSquarePixels == 0)) {
		config.output.fullWound.areaSquareCm = "[Unknown]";
		
	} else {
		config.output.fullWound.areaSquareCm = totalWoundSquareCm.toFixed(3);
	}
	
	try {
		if((totalUnderSquarePixels > 0)&&(totalStickerSquarePixels != 0)) {
			//We have some undermining - also set the wound version of these, and then create a total

			//First need to re-calculate the under area based on subtracting any area from the wound area that overlaps (i.e. the intersection with the wound area)
			if((totalWoundSquarePixels !=0) && (totalWoundSquarePixels > 0)) {
				//Find the intersection

				//Convert arrays
				var poly1 = [];
				var poly2 = [];
				var woundArray = [];
				var underArray = [];
				if(verbose == true) console.log("Wound layer: " + woundLayer + " Under Layer: " + underLayer);
			
				//Add all of the polygons to the master polys
				for(var woundPoly = 0; woundPoly < woundLayer.length; woundPoly++) {
					woundArray.push(config.output.areas[woundLayer[woundPoly]].points);
				}
				for(var underPoly = 0; underPoly < underLayer.length; underPoly++) {
					underArray.push(config.output.areas[underLayer[underPoly]].points);
				}
			
				var poly1 = polygon(woundArray);
				var poly2 = polygon(underArray);
			

				poly1.geometry.type = "Polygon";
				poly2.geometry.type = "Polygon";
				if(verbose == true) console.log("Poly1:" + JSON.stringify(poly1) + " \n\nPoly2:" + JSON.stringify(poly2));

			
					var intersectedPoly = intersect(poly1, poly2);
			
					if(intersectedPoly) {
						if(verbose == true) console.log("Intersected Poly:" + JSON.stringify(intersectedPoly));
				
						var outIntersectedPoly = [];
						var intersectedAreaSqPixels = 0;
				
						if(intersectedPoly.geometry.coordinates.length > 1) {
							//A multi polygon
							for(var poly = 0; poly < intersectedPoly.geometry.coordinates.length; poly ++) { 
								outIntersectedPoly[poly] = [];
								for(var lcnt = 0; lcnt < intersectedPoly.geometry.coordinates[poly][0].length; lcnt++) {
									outIntersectedPoly[poly][lcnt] = { x: intersectedPoly.geometry.coordinates[poly][0][lcnt][0],
											y: intersectedPoly.geometry.coordinates[poly][0][lcnt][1] };

								}
						
								if(verbose == true) console.log("Intersected before area poly:" + JSON.stringify(outIntersectedPoly[poly]));

						
								var thisArea = area(outIntersectedPoly[poly]);
								if(thisArea) {
									intersectedAreaSqPixels += thisArea;
								}
							}
						} else {
							//Single polygon
							outIntersectedPoly = [];
							for(var lcnt = 0; lcnt < intersectedPoly.geometry.coordinates[0].length; lcnt++) {
								outIntersectedPoly[lcnt] = { x: intersectedPoly.geometry.coordinates[0][lcnt][0],
										y: intersectedPoly.geometry.coordinates[0][lcnt][1] };

							}
					
							if(verbose == true) console.log("Intersected before area poly:" + JSON.stringify(outIntersectedPoly));

					
							var thisArea = area(outIntersectedPoly);
							if(thisArea) {
								intersectedAreaSqPixels += thisArea;
							}
				
						}

				

						if(verbose == true) console.log("Intersected area pixels:" + intersectedAreaSqPixels);
						totalUnderSquarePixels = totalUnderSquarePixels - intersectedAreaSqPixels;
				
						totalUnderSquareCm = (totalUnderSquarePixels / config.output.stickerSquarePixels) * config.input.sticker.physicalAreaSquareCm;
					}
			
			}

			config.output.fullWound.areaSquarePixels = totalWoundSquarePixels + totalUnderSquarePixels;
			config.output.fullWound.areaSquareCm = totalWoundSquareCm +  totalUnderSquareCm;
			config.output.fullWound.areaSquareCm = config.output.fullWound.areaSquareCm.toFixed(3);		//Ensure 3 dp string
		
		
			config.output.fullWound.woundAreaSquarePixels = totalWoundSquarePixels;
			config.output.fullWound.woundAreaSquareCm = totalWoundSquareCm.toFixed(3);
		
			config.output.fullWound.underAreaSquarePixels = totalUnderSquarePixels;
			config.output.fullWound.underAreaSquareCm = totalUnderSquareCm.toFixed(3);
		} else {
			//Otherwise clear these off
			config.output.fullWound.woundAreaSquarePixels = "";
			config.output.fullWound.woundAreaSquareCm = "";
		
			config.output.fullWound.underAreaSquarePixels = "";
			config.output.fullWound.underAreaSquareCm = "";
		
		}
	
	}		//End of intersected poly try
	catch(err) {		//There was a problem with the saving process
		config.output.fullWound.woundAreaSquarePixels = "";
		config.output.fullWound.woundAreaSquareCm = "";
		
		config.output.fullWound.underAreaSquarePixels = "";
		config.output.fullWound.underAreaSquareCm = "";
		config.output.fullWound.areaSquareCm = "[Unknown - Error]";
	}



	return config;
}




module.exports.calculateIntersectionArea = function(config)
{
	
	return config;
}


module.exports.refreshThumbnail = function(photoFile, webPhotoFile, conf, cb)
{
	//Start with the larger web-view version of the image, draw transparent polygons around
	//the wound polygons, and polygons around the sticker in a different colour,
	//and resize the image to thumbnail size and write out over the top of the last thumbnail.
	//This function is called from both a wound generation and after a save operation.


	// (B)lue, (G)reen, (R)ed, Transparency
	var stickerColour = [0, 255, 0, 90];		//Was 50 transparency		
	var woundColour = [0, 0, 255, 90];
	var underColour = [43, 80, 98, 90];		//rgb(98, 80, 43)
	var negativeColour = [98, 48, 80, 90];  //RGB: 80, 43, 98
	var freestyleColour = [255, 255, 255, 0];  //RGB: 240, 240, 240
	var lineThickness = 1;


	/*var GREEN = [0, 255, 0]; // B, G, R
		var WHITE = [255, 255, 255]; // B, G, R
		var BLACK = [0, 0, 0]; // B, G, R
		var RED   = [0, 0, 255]; // B, G, R
		var BRIGHTRED = [153, 153, 255]; // B, G, R */

	if(verbose == true) console.log("Reading web photos file: " + webPhotoFile);

	//Read in the 1200 pixel photo file
	cv.readImage(webPhotoFile, function(err, webPhoto){

		var width = webPhoto.width();
		var height = webPhoto.height();


		if (width < 1 || height < 1) throw new Error("Sorry, the image " + webPhotoFile + " needs to have a valid size. Width =" + width + " Height=" + height);

 		var polyBuff = new cv.Matrix(width, height);		//Create a buffer for the polygons, which we will blend later
 		polyBuff = webPhoto.copy();
 		polyBuff.empty();


		//Go through each polygon area
		var opencvPolySticker = [];
		var opencvPolyWound = [];
		var opencvPolyUnder = [];
		var opencvPolyNegative = [];
		var opencvPolyFreestyle = [];
		if(!conf.output.areas) conf.output.areas = [];



		for(var cnt = 0; cnt< conf.output.areas.length; cnt++) {
			var points = conf.output.areas[cnt].points;

			var arrayHere = [];
			var fromXY = [ points[0][0], height - points[0][1] ];		//Get first point
			var firstXY = fromXY;

			switch(conf.output.areas[cnt].type) {
				
				case "sticker":
					var sticker = true;
					var colour = stickerColour;
					var thislineThickness = lineThickness;
				break;
				
				case "wound":
					var sticker = false;
					var colour = woundColour;
					var thislineThickness = lineThickness;
				break;
				
				case "under":
					var sticker = false;
					var colour = underColour;
					var thislineThickness = lineThickness;
				break;
				
				case "negative":
					var sticker = false;
					var colour = negativeColour;
					var thislineThickness = lineThickness;
				break;
				
				case "freestyle":
					var sticker = false;
					var colour = freestyleColour;
					var thislineThickness = 10;
			
			}


			for(var pcnt = 0; pcnt < points.length; pcnt++) {
				var point = [ points[pcnt][0], height - points[pcnt][1] ];
				toXY = point;

				webPhoto.line(fromXY, toXY, colour, thislineThickness);
				fromXY = toXY;

				arrayHere.push(point);
			}

			if(conf.output.areas[cnt].type != "freestyle") {
				//Finish off with a final line ('freestyle' doesn't want a full polygon but a polyline, so don't include this last line)
				webPhoto.line(fromXY, firstXY, colour, thislineThickness);
			}

			switch(conf.output.areas[cnt].type) {
				
				case "sticker":
					opencvPolySticker.push(arrayHere);
				break;
				
				case "wound":
					opencvPolyWound.push(arrayHere);
				break;
				
				case "under":
					opencvPolyUnder.push(arrayHere);
				break;
				
				case "negative":
					opencvPolyNegative.push(arrayHere);
				break;
				
				case "freestyle":
					opencvPolyFreestyle.push(arrayHere);
				break;
			
			}

			


		}
		if(verbose == true) console.log("Wound polys on thumbnail:" + JSON.stringify(opencvPolyWound));
		if(verbose == true) console.log("Sticker polys on thumbnail:" + JSON.stringify(opencvPolySticker));
		if(opencvPolyWound) {
			//No need for a red background polygon: polyBuff.fillPoly(opencvPolyWound, woundColour);
		}
		if(opencvPolySticker) {
			//Make yellow stickers green - like a traffic light!
			//Only show outlines now. Was: polyBuff.fillPoly(opencvPolySticker, stickerColour);
		}
		if(opencvPolyUnder) {
			//No need for a brown background polygon: polyBuff.fillPoly(opencvPolyUnder, underColour);
		}
		if(opencvPolyNegative) {
			//Only show outlines now. Was: polyBuff.fillPoly(opencvPolyNegative, negativeColour);
		}
		
		if(opencvPolyFreestyle) {
			//Only show outlines now. 
		}

		var outBuff = new cv.Matrix(width, height);

		//Now blend in with some alpha transparency the buffer to the webphoto
		//outBuff.addWeighted(webPhoto, 0.15, polyBuff, 0.85);  //0.8 to 0.2 was good for high vis.
		
		outBuff.addWeighted(webPhoto, 0.5, polyBuff, 0.5);  //0.8 to 0.2 was good for high vis.



		 //And create a thumbnail
		  var thumbnailPhoto = new cv.Matrix(conf.output.scaledPhoto.width, conf.output.scaledPhoto.height);
		  thumbnailPhoto = outBuff.copy();		//Copy of the original 1200 pixel image
		  delete webPhoto;
		  delete outBuff;
		  var thumbnailPhotoFile = photoFile.replace(conf.output.scaledPhoto.nameConvention.replaceText,
												conf.output.scaledPhoto.nameConvention.withThumbnailText);

		  thumbnailPhoto.resize(conf.output.scaledPhoto.thumbnailWidth, conf.output.scaledPhoto.thumbnailHeight);
		  console.log('Saving thumbnail photo...' + thumbnailPhotoFile);
		  thumbnailPhoto.save(thumbnailPhotoFile);
		  delete thumbnailPhoto;
		  if(verbose == true) console.log('Done');

		  console.log('All images saved.');

		  cb(null, thumbnailPhotoFile);
	})

	return;
}
